"use strict";

const sensoreTemplate = {
    nome : "",
    stato : false,
    utilizzo : "",
};

class Sensori {

    static container = document.getElementById("sensoriContainer");

    static addHereSensori;

    static sensoriBucketIn = [];
    static sensoriBucketOut = [];

    static sensoriDisponibiliIn = [];
    static sensoriDisponibiliOut = [];

    constructor() {
        Sensori.populateSensoriDisponibili();
        Sensori.addHereSensori = Sensori.container.querySelector('ul');
        //FIXME remove this test
        /*const sens0 = Sensori.createSensoreDaValori("in0",true,"Sensore in0",true);
        const sens1 = Sensori.createSensoreDaValori("in1",false,"Sensore in1",true);
        const sens2 = Sensori.createSensoreDaValori("out",false,"Sensore out",true);*/
    }

    static populateOptions(selectNode,out = false){
        const sensori = out ? Sensori.sensoriDisponibiliOut : Sensori.sensoriDisponibiliIn;
        selectNode.innerHTML = '';
        const option = document.createElement("option");
        option.value = 'unselected';
        option.innerHTML = 'Scegli una porta di attivazione';
        selectNode.appendChild(option);
        for(const s of sensori){
            const option = document.createElement("option");
            option.value = s.nome;
            option.innerHTML = s.nome;
            selectNode.appendChild(option);
        }
    }

    static removeSensoreOption(selectNode,nome,showSensor = false,stato,utilizzo){
        if(nome == null){
            throw new Error("Nome sensore non impostato");
        }
        const options = selectNode.querySelectorAll("option");
        let removed = false;
        options.forEach(option => {
            if(option.value === nome){
                selectNode.removeChild(option);
                removed = true;
            }
        });
        if(!removed){
            Sensori.liberaSensore(nome);
            throw new Error("Errore impossibile");
        }
        if(showSensor){
            Sensori.showSensore({nome,stato,utilizzo});
            return;
        }
        // move sensor from sensoriDisponibili to sensoriBucket
        let sensore = null;
        if(nome.startsWith("in")){
            const index = Sensori.sensoriDisponibiliIn.findIndex(s => s.nome === nome);
            if(index === -1){
                Sensori.liberaSensore(nome);
                throw new Error("Errore impossibile");
            }
            sensore = Sensori.sensoriDisponibiliIn.splice(index,1)[0];
            Sensori.sensoriBucketIn.push(sensore);
        } else {
            const index = Sensori.sensoriDisponibiliOut.findIndex(s => s.nome === nome);
            if(index === -1){
                Sensori.liberaSensore(nome);
                throw new Error("Errore impossibile");
            }
            sensore = Sensori.sensoriDisponibiliOut.splice(index,1)[0]
            Sensori.sensoriBucketOut.push(sensore);
        }
        // console.log(sensore)
        return sensore;
    }

    static liberaSensore(nome){
        // the sensor is removed from sensoriDisponibili but isn't yet in sensoriBucket
        const sensore = Sensori.createSensoreDaValori(nome);
        nome.startsWith("in") ? Sensori.sensoriDisponibiliIn.push(sensore) : Sensori.sensoriDisponibiliOut.push(sensore);
    }

    static showSensore(sensore) {
        sensore.nome = sensore.nome.toLowerCase().trim();
        if(!Sensori.checkNomeSensore(sensore.nome))
            throw new Error("Nome sensore non valido");
        if(sensore.stato == null || typeof sensore.stato !== "boolean")
            throw new Error("Stato sensore non valido");
        sensore.utilizzo = sensore.utilizzo.trim();
        if(sensore.utilizzo == null || sensore.utilizzo.length === 0)
            throw new Error("Utilizzo sensore non impostato");
        
        // if(!fromRemove){
            // remove the sensor with the same name from sensoriDisponibili
            // then add the new sensor to sensoriBucket
            if(sensore.nome.startsWith("in")){
                const index = Sensori.sensoriDisponibiliIn.findIndex(s => s.nome === sensore.nome);
                if(index === -1)
                    throw new Error("Sensore non disponibile");
                Sensori.sensoriDisponibiliIn.splice(index,1);
                Sensori.sensoriBucketIn.push(sensore);
            } else {
                const index = Sensori.sensoriDisponibiliOut.findIndex(s => s.nome === sensore.nome);
                if(index === -1)
                    throw new Error("Sensore non disponibile");
                Sensori.sensoriDisponibiliOut.splice(index,1);
                Sensori.sensoriBucketOut.push(sensore);
            }
        // }
        const sensoreEl = Sensori.createSensoreHTML(sensore);
        Sensori.addHereSensori.appendChild(sensoreEl);
        Sensori.initializaTooltips();
    }

    static createSensoreHTML(sensore){
        // must set: id="sensore-in0" src="res/led-green.png" alt="sensore in0" class="" title="More info here!"
        const sensoreEl = document.createElement("li");
        sensoreEl.classList.add('d-flex','flex-column');
        sensoreEl.innerHTML = `
        <!--<li class="d-flex flex-column">-->
            <span class="d-flex justify-content-center">
                <img id="sensore-${sensore.nome}" src="res/led-${sensore.stato ? 'green' : 'red'}.png" alt="sensore-${sensore.nome}" title="${sensore.utilizzo}" height="70" width="70" data-bs-toggle="tooltip" data-bs-placement="bottom">
            </span>
            <p class="d-flex justify-content-center">${sensore.nome}</p>
        <!--</li>-->
        `;
        return sensoreEl;
    }
    /**
     * creates a new sensor and returns it. It can also be added to the bucket.
     * @param {*} nome 
     * @param {*} stato 
     * @param {*} utilizzo 
     * @param {*} createSensore 
     * @returns 
     */
    static createSensoreDaValori(nome,stato,utilizzo,createSensore = false){
        const sensore = Object.assign({}, sensoreTemplate);
        sensore.nome = nome;
        sensore.stato = stato ? stato : false;
        sensore.utilizzo = utilizzo ? utilizzo : (nome.startsWith("in") ? "Sensore in" : "Sensore out");
        if(createSensore)
            Sensori.showSensore(sensore);
        return sensore;
    }

    static checkNomeSensore(nome) {
        if(Sensori.sensoriBucketIn.length + Sensori.sensoriBucketOut.length >= 16 || Sensori.sensoriDisponibiliIn.length + Sensori.sensoriDisponibiliOut === 0)
            return false;
        if(nome == null || nome.length === 0)
            return false;
        if(nome.startsWith("in")){
            const number = nome.substring(2);
            if(number == null || isNaN(number) || isNaN(parseInt(number)) || number < 0 || number > 8)
                return false;
            for(const sensore of Sensori.sensoriBucketIn)
                if(sensore.nome === nome)
                    return false;
            return Sensori.sensoriDisponibiliIn.length > 0;
        } else if(nome.startsWith("out")){
            const number = nome.substring(3);
            if(number == null || isNaN(number) || isNaN(parseInt(number)) || number < 0 || number > 8)
                return false;
            for(const sensore of Sensori.sensoriBucketOut)
                if(sensore.nome === nome)
                    return false;
            return Sensori.sensoriDisponibiliOut.length > 0;
        } else return false;
    }

    static populateSensoriDisponibili() {
        for(let i = 0; i < 16; i++) {
            if(i < 8){
                Sensori.sensoriDisponibiliIn.push(Sensori.createSensoreDaValori("in"+i));
            } else {
                Sensori.sensoriDisponibiliOut.push(Sensori.createSensoreDaValori("out"+(i%8)));
            }
        }
    }

    static initializaTooltips() {//from bootstrap, mandatory
        const tooltipTriggerList = [].slice.call(document.querySelectorAll('[data-bs-toggle="tooltip"]'));
        const tooltipList = tooltipTriggerList.map(function (tooltipTriggerEl) {
            return new bootstrap.Tooltip(tooltipTriggerEl)
        });
    }
}

export default Sensori;